import * as i0 from '@angular/core';
import { Component, Input, NgModule } from '@angular/core';
import * as i1 from '@angular/common';
import { CommonModule } from '@angular/common';

/**
 * Creates an array of chunk objects representing both higlightable and non highlightable pieces of text that match each search word.
 * @return Array of "chunks" (where a Chunk is { start:number, end:number, highlight:boolean })
 */
const findAll = ({ autoEscape, caseSensitive = false, findChunks = defaultFindChunks, sanitize, searchWords, textToHighlight, }) => fillInChunks({
    chunksToHighlight: combineChunks({
        chunks: findChunks({
            autoEscape,
            caseSensitive,
            sanitize,
            searchWords,
            textToHighlight,
        }),
    }),
    totalLength: textToHighlight ? textToHighlight.length : 0,
});
/**
 * Takes an array of {start:number, end:number} objects and combines chunks that overlap into single chunks.
 * @return {start:number, end:number}[]
 */
const combineChunks = ({ chunks }) => {
    // @ts-ignore
    return chunks
        .sort((first, second) => first.start - second.start)
        .reduce((processedChunks, nextChunk) => {
        // First chunk just goes straight in the array...
        if (processedChunks.length === 0) {
            return [nextChunk];
        }
        else {
            // ... subsequent chunks get checked to see if they overlap...
            const prevChunk = processedChunks.pop();
            if (prevChunk && nextChunk.start <= prevChunk.end) {
                // It may be the case that prevChunk completely surrounds nextChunk, so take the
                // largest of the end indexes.
                const endIndex = Math.max(prevChunk.end, nextChunk.end);
                processedChunks.push({
                    highlight: false,
                    start: prevChunk.start,
                    end: endIndex,
                });
            }
            else {
                // @ts-ignore
                processedChunks.push(prevChunk, nextChunk);
            }
            return processedChunks;
        }
    }, []);
};
/**
 * Examine text for any matches.
 * If we find matches, add them to the returned array as a "chunk" object ({start:number, end:number}).
 * @return {start:number, end:number}[]
 */
const defaultFindChunks = ({ autoEscape, caseSensitive, sanitize = defaultSanitize, searchWords, textToHighlight, }) => {
    textToHighlight = sanitize(textToHighlight);
    return searchWords
        .filter((searchWord) => searchWord) // Remove empty words
        .reduce((chunks, searchWord) => {
        searchWord = sanitize(searchWord);
        if (autoEscape) {
            searchWord = escapeRegExpFn(searchWord);
        }
        const regex = new RegExp(searchWord, caseSensitive ? "g" : "gi");
        let match;
        while ((match = regex.exec(textToHighlight))) {
            let start = match.index;
            let end = regex.lastIndex;
            // We do not return zero-length matches
            if (end > start) {
                // @ts-ignore
                chunks.push({ highlight: false, start, end });
            }
            // Prevent browsers like Firefox from getting stuck in an infinite loop
            // See http://www.regexguru.com/2008/04/watch-out-for-zero-length-matches/
            if (match.index === regex.lastIndex) {
                regex.lastIndex++;
            }
        }
        return chunks;
    }, []);
};
/**
 * Given a set of chunks to highlight, create an additional set of chunks
 * to represent the bits of text between the highlighted text.
 * @param chunksToHighlight {start:number, end:number}[]
 * @param totalLength number
 * @return {start:number, end:number, highlight:boolean}[]
 */
const fillInChunks = ({ chunksToHighlight, totalLength, }) => {
    const allChunks = [];
    const append = (start, end, highlight) => {
        if (end - start > 0) {
            allChunks.push({
                start,
                end,
                highlight,
            });
        }
    };
    if (chunksToHighlight.length === 0) {
        append(0, totalLength, false);
    }
    else {
        let lastIndex = 0;
        chunksToHighlight.forEach((chunk) => {
            append(lastIndex, chunk.start, false);
            append(chunk.start, chunk.end, true);
            lastIndex = chunk.end;
        });
        append(lastIndex, totalLength, false);
    }
    return allChunks;
};
function defaultSanitize(string) {
    return string;
}
function escapeRegExpFn(string) {
    return string.replace(/[\-\[\]\/\{\}\(\)\*\+\?\.\\\^\$\|]/g, "\\$&");
}

class NgxHighlightWordsComponent {
    constructor() {
        this.textToHighlight = "";
        this.searchWords = [];
        this.highlightClassName = "highlight";
        this.autoEscape = true;
        this.caseSensitive = false;
        this.findChunks = defaultFindChunks;
        this.sanitize = defaultSanitize;
    }
    get chunks() {
        return findAll({
            textToHighlight: this.textToHighlight,
            searchWords: this.searchWords,
            autoEscape: this.autoEscape,
            caseSensitive: this.caseSensitive,
            findChunks: this.findChunks,
            sanitize: this.sanitize,
        });
    }
}
NgxHighlightWordsComponent.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "12.2.13", ngImport: i0, type: NgxHighlightWordsComponent, deps: [], target: i0.ɵɵFactoryTarget.Component });
NgxHighlightWordsComponent.ɵcmp = i0.ɵɵngDeclareComponent({ minVersion: "12.0.0", version: "12.2.13", type: NgxHighlightWordsComponent, selector: "ngx-highlight-words", inputs: { textToHighlight: "textToHighlight", searchWords: "searchWords", highlightClassName: "highlightClassName", autoEscape: "autoEscape", caseSensitive: "caseSensitive", findChunks: "findChunks", sanitize: "sanitize" }, ngImport: i0, template: "<span\n  [ngClass]=\"chunk.highlight ? highlightClassName : ''\"\n  *ngFor=\"let chunk of chunks\"\n  >{{ textToHighlight.substr(chunk.start, chunk.end - chunk.start) }}</span\n>\n", directives: [{ type: i1.NgForOf, selector: "[ngFor][ngForOf]", inputs: ["ngForOf", "ngForTrackBy", "ngForTemplate"] }, { type: i1.NgClass, selector: "[ngClass]", inputs: ["class", "ngClass"] }] });
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "12.2.13", ngImport: i0, type: NgxHighlightWordsComponent, decorators: [{
            type: Component,
            args: [{
                    selector: "ngx-highlight-words",
                    templateUrl: "./ngx-highlight-words.component.html",
                }]
        }], propDecorators: { textToHighlight: [{
                type: Input
            }], searchWords: [{
                type: Input
            }], highlightClassName: [{
                type: Input
            }], autoEscape: [{
                type: Input
            }], caseSensitive: [{
                type: Input
            }], findChunks: [{
                type: Input
            }], sanitize: [{
                type: Input
            }] } });

class NgxHighlightWordsModule {
}
NgxHighlightWordsModule.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "12.2.13", ngImport: i0, type: NgxHighlightWordsModule, deps: [], target: i0.ɵɵFactoryTarget.NgModule });
NgxHighlightWordsModule.ɵmod = i0.ɵɵngDeclareNgModule({ minVersion: "12.0.0", version: "12.2.13", ngImport: i0, type: NgxHighlightWordsModule, declarations: [NgxHighlightWordsComponent], imports: [CommonModule], exports: [NgxHighlightWordsComponent] });
NgxHighlightWordsModule.ɵinj = i0.ɵɵngDeclareInjector({ minVersion: "12.0.0", version: "12.2.13", ngImport: i0, type: NgxHighlightWordsModule, imports: [[CommonModule]] });
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "12.2.13", ngImport: i0, type: NgxHighlightWordsModule, decorators: [{
            type: NgModule,
            args: [{
                    declarations: [NgxHighlightWordsComponent],
                    imports: [CommonModule],
                    exports: [NgxHighlightWordsComponent],
                }]
        }] });

/*
 * Public API Surface of ngx-highlight-words
 */

/**
 * Generated bundle index. Do not edit.
 */

export { NgxHighlightWordsComponent, NgxHighlightWordsModule };
//# sourceMappingURL=ngx-highlight-words.js.map
